/*
 asm6502.h

  Revised: 2013.05.13

 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef __ASM6502_H
 #include "disasm6502.h"

/* ---- constants */

/* flg (flag) bits */
#define _Prescan	0x1	/* allow temporary auto-creation */
#define _Optimize	0x2	/* make smaller codes - a file can turn this on */

#define _UnestID	0x20	/* prescan (unestablished) ID added */
#define _EndKeyw	0x40	/* 'end' statement reached (not EOF) */
#define _IntoLine	0x80	/* otherwise start of line */

enum
{ /* type values */
	_Unestablished,
	_Label,
	_Equate
};

enum
{ /* assembler errors (other than the interpreter.h standard) */
	_e_include_depth	=-1000,
	_e_inc_open		=-1001,
	_e_on_off		=-1002
};

enum
{ /* corresponding index is +OPN_COUNT */
	_incbin		= OPN_COUNT,
	_include,
	_bank,
	_block,
	_bss,
	_code,
	_db,
	_dcb,
	_dc_b,
	_default,
	_defchr,
	_dw,
	_dcw,
	_dc_w,
	_end,
	_equ,
	_fill,
	_module,
	_off,
	_on,
	_opt,
	_org,
	_p02,
	_pad,
	_processor,
	_subroutine,
	_zeropage,
	_zp
};

/* ---- typedefs */

typedef struct _asm6502frame
{
	uint16		org;
	uint8		bnk, flg;
	/* null term'd list of: [hash.8] [pstr] [type.8] [val.32] */
	uint8		*lab;
} _asm6502frame;

/* ---- prototypes */

/* returns pstr offset (positive), ~end, zero on error */
int32 X6502_AsmGetID(const char *s,
	const char *e,
	uint8 *n, /* buffer capacity >=256 */
	const _asm6502frame *f);
/* returns data offset; negated if already added, zero on error */
int X6502_AsmAddID(const char *s,
	const char *e,
	uint8 *n, /* buffer capacity >=256 */
	_asm6502frame *f,
	uint8 t); /* see type values above */

/* returns either negative (error), positive when finished */
int X6502_Assemble(const char *s, /* C-string - must have complete lines */
	int (*writef)(uint16 addr, /* can be NULL (check syntax) */
		uint8 bank,
		uint8 v),
	_asm6502frame *f); /* can be NULL (one call); zero it out first thing */

/* returns the total parsed length, and larger operand mode in `m` */
long X6502_ParseOperands(const char *str,
	const char *e,
	evalvalue *v,
	_asm6502frame *f,
	int *m);
/* returns index */
int X6502_ParseKey(const char *s,
	const char *e,
	uint32 max);
/* returns text length parsed (positive), otherwise error */
long X6502_ParseName(const char *str,
	const char *e,
	long *z, /* out actual length */
	char **b);
#endif
